import logging
from ctypes import byref, c_uint8, c_uint16, c_uint32, CFUNCTYPE, Structure
from time import sleep, time
from typing import Optional

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class IntrusionTime(Structure):
    """Intrusion time (define in: sdk/include/lmbinc.h)."""
    _fields_ = [
        ("uw_year", c_uint16),
        ("ub_month", c_uint8),
        ("ub_day", c_uint8),
        ("ub_hour", c_uint8),
        ("ub_minute", c_uint8),
        ("ub_second", c_uint8),
    ]


class IntrusionMsg(Structure):
    """Intrusion callback function (define in: sdk/include/lmbinc.h)."""
    _fields_ = [
        ("udw_occur_item", c_uint32),
        ("udw_status", c_uint32),
        ("stu_time", IntrusionTime),
    ]


class SoftwareReset:
    """
    Software Reset.

    sdk/src_utils/sdk_sled/sdk_swr.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._ub_read = c_uint8()

    def get_status(self) -> int:
        """Get software reset button status."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SWR_GetStatus(byref(self._ub_read))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SWR_GetStatus", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"SWReset button Status is {self._ub_read.value}")
            return self._ub_read.value

    @classmethod
    def _callback(cls, stu_intrusion_msg: IntrusionMsg) -> None:
        """Callback function for exec_callback()"""
        print(f"SWR Item = {stu_intrusion_msg.udw_occur_item:04X}, "
              f"Status = {stu_intrusion_msg.udw_status:04X}, "
              f"time is {stu_intrusion_msg.stu_time.uw_year:04d}/"
              f"{stu_intrusion_msg.stu_time.ub_month:02d}/"
              f"{stu_intrusion_msg.stu_time.ub_day:02d} "
              f"{stu_intrusion_msg.stu_time.ub_hour:02d}:"
              f"{stu_intrusion_msg.stu_time.ub_minute:02d}:"
              f"{stu_intrusion_msg.stu_time.ub_second:02d}")

    def exec_callback(self) -> None:
        """Use callback function to detect software reset button status (default 10 seconds)."""
        c_callback = CFUNCTYPE(None, IntrusionMsg)
        p_callback = c_callback(self._callback)
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SWR_IntrCallback(p_callback, 150)
            if i_ret != PSP.ERR_Success:
                print("-----> hook Software-Reset button callback failure <-------")
                return
            print("----> hook Software-Reset button Callback OK <----")
            print("===> wait about 10 second time <===")
            sleep(10)
            i_ret = psp.LMB_SWR_IntrCallback(None, 150)
            if i_ret == PSP.ERR_Success:
                print("----> disabled Software-Reset button Callback hook <----")

    def test(self, seconds: int = 5) -> None:
        """For testing (default 5 seconds delay).

        :param seconds: seconds
        """
        # Check type.
        if not isinstance(seconds, int):
            raise TypeError("'seconds' type must be int")
        # Check value.
        if seconds <= 0:
            raise ValueError("'seconds' value must be > 0")
        print(f"===> wait {seconds} seconds for Software Reset Button trigger .......")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            while seconds >= 0:
                if round(seconds % 1, 1) == 0.0:
                    print(f"{int(seconds):d}. ", end="", flush=True)
                psp.LMB_SWR_GetStatus(byref(self._ub_read))
                if self._ub_read.value == 1:
                    break
                seconds -= 0.1
                sleep(0.1)
        print()
        if self._ub_read.value == 1:
            print("Software/Reset button pressed ! --> OK")
        else:
            print("\033[1;31mSoftware/Reset button not detected ! --> ALARM\033[0m")

    @property
    def is_pressed(self) -> bool:
        """Returns `True` if the device is currently active and `False` otherwise."""
        return bool(self.get_status())

    def wait_for_press(self, timeout: Optional[float] = None) -> None:
        """Pause the script until the device is activated, or the timeout is reached.

        :param timeout: Number of seconds to wait before proceeding.
            If this is `None` (the default), then wait indefinitely until the device is active.
        """
        if timeout is not None:
            if not isinstance(timeout, float):
                raise TypeError("'timeout' type must be float or None")
            if timeout <= 0.0:
                raise ValueError("'timeout' value must be > 0.0")
        start_time = time()
        while True:
            if self.is_pressed:
                break
            if timeout and time() - start_time >= timeout:
                break
            sleep(0.1)

    def wait_for_release(self, timeout: Optional[float] = None) -> None:
        """Pause the script until the device is deactivated, or the timeout is reached.

        :param timeout: Number of seconds to wait before proceeding.
            If this is `None` (the default), then wait indefinitely until the device is active.
        """
        if timeout is not None:
            if not isinstance(timeout, float):
                raise TypeError("'timeout' type must be float or None")
            if timeout <= 0.0:
                raise ValueError("'timeout' value must be > 0.0")
        start_time = time()
        while True:
            if not self.is_pressed:
                break
            if timeout and time() - start_time >= timeout:
                break
            sleep(0.1)



from .com_port import ComPort
from .lmbinc import PSP
from .sdk_gps import GPS
from .sdk_gsr import GSR
from .sdk_hwm import HWM
from .sdk_rfm import RFM
from .sdk_sled import SystemLED
from .sdk_sled_gps import GPSLED
from .sdk_sled_lte import LteStateLED
from .sdk_sled_lte_stress import LteStressLED
from .sdk_swr import SoftwareReset
from .sdk_wdt import WatchdogTimer

__version__ = "0.0.3"
__all__ = ["PSP", "ComPort", "GPS", "GSR", "HWM", "RFM", "SystemLED", "GPSLED", "LteStateLED", "LteStressLED", "SoftwareReset", "WatchdogTimer"]



import logging
from ctypes import byref, c_uint32

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class RFM:
    """
    Radio Frequency Module.

    sdk/src_utils/sdk_rfm/sdk_rfm.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._udw_reg = c_uint32(0)

    def get_module(self) -> int:
        """Get LTE Module power state.

        bit 0 represent m.2 module, bit 1 represent mPCIE module
        1: power on , 0: power off

        0 (00): mPcie -> off, m.2 -> off
        1 (01): mPcie -> off, m.2 -> on
        2 (10): mPcie -> on,  m.2 -> off
        3 (11): mPcie -> on,  m.2 -> on
        """
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_RFM_GetModule(byref(self._udw_reg))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_RFM_GetModule", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"get rfm module status {self._udw_reg.value:x}")
            return self._udw_reg.value

    def set_module(self, value: int) -> None:
        """Set LTE Module power state.

        bit 0 represent m.2 module, bit 1 represent mPCIE module
        1: power on, 0: power off

        0 (00): mPcie -> off, m.2 -> off
        1 (01): mPcie -> off, m.2 -> on
        2 (10): mPcie -> on,  m.2 -> off
        3 (11): mPcie -> on,  m.2 -> on
        """
        # Check type.
        if not isinstance(value, int):
            raise TypeError("'value' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_RFM_SetModule(value)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_RFM_SetModule", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"set rfm module status {value}")

    def get_sim(self) -> int:
        """Get SIM card state.

        bit 0 represent m.2 module, bit 1 represent mPCIE module
        0: first sim, 1: second sim

        0 (00): mPcie -> first sim (SIM3),  m.2 -> first sim (SIM1)
        1 (01): mPcie -> first sim (SIM3),  m.2 -> second sim (SIM2)
        2 (10): mPcie -> second sim (SIM4), m.2 -> first sim (SIM1)
        3 (11): mPcie -> second sim (SIM4), m.2 -> second sim (SIM2)
        """
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_RFM_GetSIM(byref(self._udw_reg))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_RFM_GetSIM", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"get rfm sim status {self._udw_reg.value:x}")
            return self._udw_reg.value

    def set_sim(self, value: int) -> None:
        """Set SIM card state.

        bit 0 represent m.2 module, bit 1 represent mPCIE module
        0: first sim, 1: second sim

        0 (00): mPcie -> first sim (SIM3),  m.2 -> first sim (SIM1)
        1 (01): mPcie -> first sim (SIM3),  m.2 -> second sim (SIM2)
        2 (10): mPcie -> second sim (SIM4), m.2 -> first sim (SIM1)
        3 (11): mPcie -> second sim (SIM4), m.2 -> second sim (SIM2)
        """
        # Check type.
        if not isinstance(value, int):
            raise TypeError("'value' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_RFM_SetSIM(value)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_RFM_SetSIM", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"set rfm sim status {value}")



import logging
from time import sleep

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class LteStateLED:
    """
    LTE Status LED.

    sdk/src_utils/sdk_sled_lte/sdk_sled_lte.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path

    def off(self) -> None:
        """Set LTE Status LED to off."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led off")

    def red(self) -> None:
        """Set LTE Status LED to red."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led red on")

    def red_blink(self) -> None:
        """Set LTE Status LED to red blink."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(2)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led red blink")

    def green(self) -> None:
        """Set LTE Status LED to green."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(3)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led green on")

    def green_blink(self) -> None:
        """Set LTE Status LED to green blink."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(4)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led green blink")

    def yellow(self) -> None:
        """Set LTE Status LED to yellow."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(5)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led yellow on")

    def yellow_blink(self) -> None:
        """Set LTE Status LED to yellow blink."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(0)  # Clear color.
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_SetLteStateLED(6)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte led yellow blink")

    def test(self, seconds: float = 2.0) -> None:
        """For testing (default 2 seconds delay).

        :param seconds: seconds
        """
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStateLED(1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led red on")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(2)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led red blink")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led off")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(3)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led green on")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(4)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led green blink")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led off")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(5)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led yellow on")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(6)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led yellow blink")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStateLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStateLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte led off")



import logging
from ctypes import c_char_p

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class GPS:
    """
    GPS.

    sdk/src_utils/sdk_gps/sdk_gps.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path

    def search_port(self) -> str:
        """Search GPS port."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            default_gps_port = "/dev/ttyS1"
            str_gps_port = c_char_p(default_gps_port.encode())
            i_ret = psp.LMB_GPS_SearchPort(str_gps_port)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_GPS_SearchPort", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            gps_port = str_gps_port.value.decode()
            logger.info(f"get gps port {gps_port}")
            return gps_port



import logging
from ctypes import byref, c_int16, Structure
from time import sleep

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class AxisRawData(Structure):
    """G-Sensor X,Y,Z Axis (define in: sdk/include/lmbinc.h)."""
    _fields_ = [
        ("w_x_axis", c_int16),
        ("w_y_axis", c_int16),
        ("w_z_axis", c_int16),
        ("wg_range", c_int16),
    ]


class AxisData:
    """To store axis data."""

    def __init__(self, wg_range: int, w_x_axis: int, w_y_axis: int, w_z_axis: int,
                 f_x_mg: float, f_y_mg: float, f_z_mg: float) -> None:
        self._wg_range = wg_range
        self._w_x_axis = w_x_axis
        self._w_y_axis = w_y_axis
        self._w_z_axis = w_z_axis
        self._f_x_mg = f_x_mg
        self._f_y_mg = f_y_mg
        self._f_z_mg = f_z_mg

    @property
    def wg_range(self) -> int:
        return self._wg_range

    @property
    def w_x_axis(self) -> int:
        return self._w_x_axis

    @property
    def w_y_axis(self) -> int:
        return self._w_y_axis

    @property
    def w_z_axis(self) -> int:
        return self._w_z_axis

    @property
    def f_x_mg(self) -> float:
        return self._f_x_mg

    @property
    def f_y_mg(self) -> float:
        return self._f_y_mg

    @property
    def f_z_mg(self) -> float:
        return self._f_z_mg


class AxisOffset:
    """To store axis offset."""

    def __init__(self, w_x_axis: int, w_y_axis: int, w_z_axis: int) -> None:
        self._w_x_axis = w_x_axis
        self._w_y_axis = w_y_axis
        self._w_z_axis = w_z_axis

    @property
    def w_x_axis(self) -> int:
        return self._w_x_axis

    @property
    def w_y_axis(self) -> int:
        return self._w_y_axis

    @property
    def w_z_axis(self) -> int:
        return self._w_z_axis


class GSR:
    """
    G-Sensor.

    sdk/src_utils/sdk_gsr/sdk_gsr.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._stu_raw_data = AxisRawData()

    def get_axis_data(self) -> AxisData:
        """Get X/Y/Z direction accelerate value."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_GSR_GetAxisData(byref(self._stu_raw_data))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_GSR_GetAxisData", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)

            logger.info(f"get gsr w-range= ±{self._stu_raw_data.wg_range:d}g")

            if self._stu_raw_data.wg_range == 2:
                fmg_step = 2 / 255
            elif self._stu_raw_data.wg_range == 4:
                fmg_step = 4 / 255
            elif self._stu_raw_data.wg_range == 8:
                fmg_step = 8 / 255
            elif self._stu_raw_data.wg_range == 16:
                fmg_step = 16 / 255

            f_x_mg = self._stu_raw_data.w_x_axis * fmg_step
            f_y_mg = self._stu_raw_data.w_y_axis * fmg_step
            f_z_mg = self._stu_raw_data.w_z_axis * fmg_step

            logger.info(f"get gsr x-axis raw= {self._stu_raw_data.w_x_axis:d}, accel= {f_x_mg:03.8f}")
            logger.info(f"get gsr x-axis raw= {self._stu_raw_data.w_y_axis:d}, accel= {f_y_mg:03.8f}")
            logger.info(f"get gsr x-axis raw= {self._stu_raw_data.w_z_axis:d}, accel= {f_z_mg:03.8f}")

            return AxisData(wg_range=self._stu_raw_data.wg_range,
                            w_x_axis=self._stu_raw_data.w_x_axis,
                            w_y_axis=self._stu_raw_data.w_y_axis,
                            w_z_axis=self._stu_raw_data.w_z_axis,
                            f_x_mg=f_x_mg,
                            f_y_mg=f_y_mg,
                            f_z_mg=f_z_mg)

    def get_axis_offset(self) -> AxisOffset:
        """Get X/Y/Z direction offset value."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_GSR_GetAxisOffset(byref(self._stu_raw_data))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_GSR_GetAxisOffset", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)

            logger.info(f"get gsr x-axis offset= {self._stu_raw_data.w_x_axis:d}")
            logger.info(f"get gsr y-axis offset= {self._stu_raw_data.w_y_axis:d}")
            logger.info(f"get gsr z-axis offset= {self._stu_raw_data.w_z_axis:d}")

            return AxisOffset(w_x_axis=self._stu_raw_data.w_x_axis,
                              w_y_axis=self._stu_raw_data.w_y_axis,
                              w_z_axis=self._stu_raw_data.w_z_axis)

    def test(self) -> None:
        """For testing."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            for i in range(100):
                logger.info(f"---------> {i:d}")

                # Get accel data.
                i_ret = psp.LMB_GSR_GetAxisData(byref(self._stu_raw_data))
                if i_ret != PSP.ERR_Success:
                    error_message = PSP.get_error_message("LMB_GSR_GetAxisData", i_ret)
                    logger.error(error_message)
                else:
                    logger.info(f"stuRawData.wRange= ±{self._stu_raw_data.wg_range:d}g")

                    if self._stu_raw_data.wg_range == 2:
                        fmg_step = 2 / 255
                    elif self._stu_raw_data.wg_range == 4:
                        fmg_step = 4 / 255
                    elif self._stu_raw_data.wg_range == 8:
                        fmg_step = 8 / 255
                    elif self._stu_raw_data.wg_range == 16:
                        fmg_step = 16 / 255

                    f_x_mg = self._stu_raw_data.w_x_axis * fmg_step
                    f_y_mg = self._stu_raw_data.w_y_axis * fmg_step
                    f_z_mg = self._stu_raw_data.w_z_axis * fmg_step

                    logger.info(f"Raw={self._stu_raw_data.w_x_axis:d}\t, X-Axis= {f_x_mg:03.8f}")
                    logger.info(f"Raw={self._stu_raw_data.w_y_axis:d}\t, Y-Axis= {f_y_mg:03.8f}")
                    logger.info(f"Raw={self._stu_raw_data.w_z_axis:d}\t, Z-Axis= {f_z_mg:03.8f}")

                # Get offset.
                i_ret = psp.LMB_GSR_GetAxisOffset(byref(self._stu_raw_data))
                if i_ret != PSP.ERR_Success:
                    error_message = PSP.get_error_message("LMB_GSR_GetAxisOffset", i_ret)
                    logger.error(error_message)
                else:
                    logger.info(f"Offset X-Axis={self._stu_raw_data.w_x_axis:d}")
                    logger.info(f"Offset Y-Axis={self._stu_raw_data.w_y_axis:d}")
                    logger.info(f"Offset Z-Axis={self._stu_raw_data.w_z_axis:d}")

                sleep(0.5)



import logging
from configparser import ConfigParser
from ctypes import byref, c_float, c_uint16

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class HWM:
    """
    Hardware Monitor.

    sdk/src_utils/sdk_hwm/sdk_hwm.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._f_temp = c_float()
        self._w_data = c_uint16()
        self._w_rpm = c_uint16()

    @classmethod
    def _str_replace(cls, source: str) -> float:
        """Replace str to float from hwm.conf"""
        result = 1.0
        for element in source.split("*"):
            result *= float(element.strip())
        return result

    def get_cpu_temp(self, num: int) -> int:
        """Get CPU temperature."""
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_GetCpuTemp(num, byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_GetCpuTemp", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"CPU-{num} temperature = {int(self._f_temp.value):d}")
            return int(self._f_temp.value)

    def get_sys_temp(self, num: int) -> int:
        """Get SYS temperature."""
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_GetSysTemp(num, byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_GetSysTemp", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"SYS-{num} temperature = {int(self._f_temp.value):d}")
            return int(self._f_temp.value)

    def get_vcore(self, num: int) -> float:
        """Get CPU core voltage."""
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_GetVcore(num, byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_GetVcore", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"CPU-{num} Vcore = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_12v(self) -> float:
        """Get 12V voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_Get12V(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_Get12V", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"12V = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_5v(self) -> float:
        """Get 5V voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_Get5V(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_Get5V", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"5V = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_3v3(self) -> float:
        """Get 3.3V voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_Get3V3(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_Get3V3", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"3.3V = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_5vsb(self) -> float:
        """Get 5Vsb voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_Get5Vsb(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_Get5Vsb", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"5VSB = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_3v3sb(self) -> float:
        """Get 3.3Vsb voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_Get3V3sb(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_Get3V3sb", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"3.3VSB = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_vbat(self) -> float:
        """Get Vbat voltage."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_GetVbat(byref(self._f_temp))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_GetVbat", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"Vbat = {self._f_temp.value:2.3f}")
            return self._f_temp.value

    def get_power_supply(self, num: int) -> int:
        """Get Power Supply voltage."""
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_HWM_GetPowerSupply(num, byref(self._w_data))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_HWM_GetPowerSupply", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"PowerSupply {num} AC voltage = {self._f_temp.value:d}")
            return self._w_data.value

    def testhwm(self, conf_path: str = "/opt/lanner/psp/bin/amd64/utils/hwm.conf") -> None:
        """For hardware monitor testing."""
        cp = ConfigParser()
        cp.read(conf_path)
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            # Temperature.
            if psp.LMB_HWM_GetCpuTemp(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU1_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_CPU1_Temp"]["max"])
                logger.info(f"CPU-1 temperature = {int(self._f_temp.value):3d} C\t"
                            f"(min = {min_:3.0f} C, max = {max_:3.0f} C)")
            if psp.LMB_HWM_GetCpuTemp(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU2_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_CPU2_Temp"]["max"])
                logger.info(f"CPU-2 temperature = {int(self._f_temp.value):3d} C\t"
                            f"(min = {min_:3.0f} C, max = {max_:3.0f} C)")
            if psp.LMB_HWM_GetSysTemp(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS1_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_SYS1_Temp"]["max"])
                logger.info(f"SYS-1 temperature = {int(self._f_temp.value):3d} C\t"
                            f"(min = {min_:3.0f} C, max = {max_:3.0f} C)")
            if psp.LMB_HWM_GetSysTemp(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS2_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_SYS2_Temp"]["max"])
                logger.info(f"SYS-2 temperature = {int(self._f_temp.value):3d} C\t"
                            f"(min = {min_:3.0f} C, max = {max_:3.0f} C)")
            # Voltage.
            if psp.LMB_HWM_GetVcore(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_Core1_volt"]["min"])
                max_ = self._str_replace(cp["HWM_Core1_volt"]["max"])
                logger.info(f"CPU-1 Vcore = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_GetVcore(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_Core2_volt"]["min"])
                max_ = self._str_replace(cp["HWM_Core2_volt"]["max"])
                logger.info(f"CPU-2 Vcore = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_Get12V(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_12v_volt"]["min"])
                max_ = self._str_replace(cp["HWM_12v_volt"]["max"])
                logger.info(f"12V = {self._f_temp.value:7.3f} V\t\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_Get5V(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_5v_volt"]["min"])
                max_ = self._str_replace(cp["HWM_5v_volt"]["max"])
                logger.info(f"5V = {self._f_temp.value:7.3f} V\t\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_Get3V3(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_3v3_volt"]["min"])
                max_ = self._str_replace(cp["HWM_3v3_volt"]["max"])
                logger.info(f"3.3V = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_Get5Vsb(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_5vsb_volt"]["min"])
                max_ = self._str_replace(cp["HWM_5vsb_volt"]["max"])
                logger.info(f"5VSB = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_Get3V3sb(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_3v3sb_volt"]["min"])
                max_ = self._str_replace(cp["HWM_3v3sb_volt"]["max"])
                logger.info(f"3.3VSB = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_GetVbat(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_vBat_volt"]["min"])
                max_ = self._str_replace(cp["HWM_vBat_volt"]["max"])
                logger.info(f"Vbat = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_GetVDDR(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_vddr_volt"]["min"])
                max_ = self._str_replace(cp["HWM_vddr_volt"]["max"])
                logger.info(f"VDDR = {self._f_temp.value:7.3f} V\t\t"
                            f"(min = {min_:7.3f} V, max = {max_:7.3f} V)")
            if psp.LMB_HWM_GetPowerSupply(1, byref(self._w_data)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_PSU1_volt"]["min"])
                max_ = self._str_replace(cp["HWM_PSU1_volt"]["max"])
                logger.info(f"PowerSupply 1 AC voltage = {self._w_data.value:3d} V\t"
                            f"(min = {min_:3.0f} V, max = {max_:3.0f} V)")
            if psp.LMB_HWM_GetPowerSupply(2, byref(self._w_data)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_PSU2_volt"]["min"])
                max_ = self._str_replace(cp["HWM_PSU2_volt"]["max"])
                logger.info(f"PowerSupply 2 AC voltage = {self._w_data.value:3d} V\t"
                            f"(min = {min_:3.0f} V, max = {max_:3.0f} V)")
            # Fan RPM.
            if psp.LMB_HWM_GetCpuFan(1, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU1_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_CPU1_RPM"]["max"])
                logger.info(f"CPU FAN 1 speed = {self._w_rpm.value:5d} rpm\t"
                            f"(min = {min_:5.0f} rpm, max = {max_:5.0f} rpm)")
            if psp.LMB_HWM_GetCpuFan(2, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU2_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_CPU2_RPM"]["max"])
                logger.info(f"CPU FAN 2 speed = {self._w_rpm.value:5d} rpm\t"
                            f"(min = {min_:5.0f} rpm, max = {max_:5.0f} rpm)")
            if psp.LMB_HWM_GetSysFan(1, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS1_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_SYS1_RPM"]["max"])
                logger.info(f"SYS FAN 1 speed = {self._w_rpm.value:5d} rpm\t"
                            f"(min = {min_:5.0f} rpm, max = {max_:5.0f} rpm)")
            if psp.LMB_HWM_GetSysFan(2, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS2_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_SYS2_RPM"]["max"])
                logger.info(f"SYS FAN 2 speed = {self._w_rpm.value:5d} rpm\t"
                            f"(min = {min_:5.0f} rpm, max = {max_:5.0f} rpm)")

    def get_all(self, conf_path: str = "/opt/lanner/psp/bin/amd64/utils/hwm.conf") -> dict:
        """Get all exist value to dict."""
        cp = ConfigParser()
        cp.read(conf_path)
        data = {}
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            # Temperature.
            if psp.LMB_HWM_GetCpuTemp(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU1_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_CPU1_Temp"]["max"])
                data["cpu_temp_1"] = {"min": int(f"{min_:3.0f}"),
                                      "max": int(f"{max_:3.0f}"),
                                      "current": int(self._f_temp.value)}
            if psp.LMB_HWM_GetCpuTemp(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU2_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_CPU2_Temp"]["max"])
                data["cpu_temp_2"] = {"min": int(f"{min_:3.0f}"),
                                      "max": int(f"{max_:3.0f}"),
                                      "current": int(self._f_temp.value)}
            if psp.LMB_HWM_GetSysTemp(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS1_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_SYS1_Temp"]["max"])
                data["sys_temp_1"] = {"min": int(f"{min_:3.0f}"),
                                      "max": int(f"{max_:3.0f}"),
                                      "current": int(self._f_temp.value)}
            if psp.LMB_HWM_GetSysTemp(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS2_Temp"]["min"])
                max_ = self._str_replace(cp["HWM_SYS2_Temp"]["max"])
                data["sys_temp_2"] = {"min": int(f"{min_:3.0f}"),
                                      "max": int(f"{max_:3.0f}"),
                                      "current": int(self._f_temp.value)}
            # Voltage.
            if psp.LMB_HWM_GetVcore(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_Core1_volt"]["min"])
                max_ = self._str_replace(cp["HWM_Core1_volt"]["max"])
                data["vcore_1"] = {"min": float(f"{min_:7.3f}"),
                                   "max": float(f"{max_:7.3f}"),
                                   "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_GetVcore(2, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_Core2_volt"]["min"])
                max_ = self._str_replace(cp["HWM_Core2_volt"]["max"])
                data["vcore_2"] = {"min": float(f"{min_:7.3f}"),
                                   "max": float(f"{max_:7.3f}"),
                                   "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_Get12V(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_12v_volt"]["min"])
                max_ = self._str_replace(cp["HWM_12v_volt"]["max"])
                data["12v"] = {"min": float(f"{min_:7.3f}"),
                               "max": float(f"{max_:7.3f}"),
                               "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_Get5V(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_5v_volt"]["min"])
                max_ = self._str_replace(cp["HWM_5v_volt"]["max"])
                data["5v"] = {"min": float(f"{min_:7.3f}"),
                              "max": float(f"{max_:7.3f}"),
                              "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_Get3V3(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_3v3_volt"]["min"])
                max_ = self._str_replace(cp["HWM_3v3_volt"]["max"])
                data["3v3"] = {"min": float(f"{min_:7.3f}"),
                               "max": float(f"{max_:7.3f}"),
                               "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_Get5Vsb(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_5vsb_volt"]["min"])
                max_ = self._str_replace(cp["HWM_5vsb_volt"]["max"])
                data["5vsb"] = {"min": float(f"{min_:7.3f}"),
                                "max": float(f"{max_:7.3f}"),
                                "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_Get3V3sb(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_3v3sb_volt"]["min"])
                max_ = self._str_replace(cp["HWM_3v3sb_volt"]["max"])
                data["3v3sb"] = {"min": float(f"{min_:7.3f}"),
                                 "max": float(f"{max_:7.3f}"),
                                 "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_GetVbat(byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_vBat_volt"]["min"])
                max_ = self._str_replace(cp["HWM_vBat_volt"]["max"])
                data["vbat"] = {"min": float(f"{min_:7.3f}"),
                                "max": float(f"{max_:7.3f}"),
                                "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_GetVDDR(1, byref(self._f_temp)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_vddr_volt"]["min"])
                max_ = self._str_replace(cp["HWM_vddr_volt"]["max"])
                data["vddr"] = {"min": float(f"{min_:7.3f}"),
                                "max": float(f"{max_:7.3f}"),
                                "current": float(f"{self._f_temp.value:7.3f}")}
            if psp.LMB_HWM_GetPowerSupply(1, byref(self._w_data)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_PSU1_volt"]["min"])
                max_ = self._str_replace(cp["HWM_PSU1_volt"]["max"])
                data["power_supply_1"] = {"min": int(f"{min_:3.0f}"),
                                          "max": int(f"{max_:3.0f}"),
                                          "current": int(self._w_data.value)}
            if psp.LMB_HWM_GetPowerSupply(2, byref(self._w_data)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_PSU2_volt"]["min"])
                max_ = self._str_replace(cp["HWM_PSU2_volt"]["max"])
                data["power_supply_2"] = {"min": int(f"{min_:3.0f}"),
                                          "max": int(f"{max_:3.0f}"),
                                          "current": int(self._w_data.value)}
            # Fan RPM.
            if psp.LMB_HWM_GetCpuFan(1, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU1_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_CPU1_RPM"]["max"])
                data["cpu_fan_1"] = {"min": int(f"{min_:5.0f}"),
                                     "max": int(f"{max_:5.0f}"),
                                     "current": int(self._w_rpm.value)}
            if psp.LMB_HWM_GetCpuFan(2, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_CPU2_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_CPU2_RPM"]["max"])
                data["cpu_fan_2"] = {"min": int(f"{min_:5.0f}"),
                                     "max": int(f"{max_:5.0f}"),
                                     "current": int(self._w_rpm.value)}
            if psp.LMB_HWM_GetSysFan(1, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS1_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_SYS1_RPM"]["max"])
                data["sys_fan_1"] = {"min": int(f"{min_:5.0f}"),
                                     "max": int(f"{max_:5.0f}"),
                                     "current": int(self._w_rpm.value)}
            if psp.LMB_HWM_GetSysFan(2, byref(self._w_rpm)) == PSP.ERR_Success:
                min_ = self._str_replace(cp["HWM_SYS2_RPM"]["min"])
                max_ = self._str_replace(cp["HWM_SYS2_RPM"]["max"])
                data["sys_fan_2"] = {"min": int(f"{min_:5.0f}"),
                                     "max": int(f"{max_:5.0f}"),
                                     "current": int(self._w_rpm.value)}
            return data



import logging
from time import sleep

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class LteStressLED:
    """
    LTE Stress LED.

    sdk/src_utils/sdk_sled_lte_stress/sdk_sled_lte_stress.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path

    def off(self) -> None:
        """Set LTE Stress LED to off."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStressLED(-1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("set lte stress led off")

    def set_strength(self, percent: int) -> None:
        """Set LTE Stress LED signal strength."""
        # Check type.
        if not isinstance(percent, int):
            raise TypeError("'percent' type must be int")
        # Check value.
        if not 0 <= percent <= 100:
            raise ValueError("'percent' value must be between 0 and 100")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStressLED(percent)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"set lte stress led {percent}%")

    def test(self, seconds: float = 2.0) -> None:
        """For testing (default 2 seconds delay).

        :param seconds: seconds
        """
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetLteStressLED(90)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 8")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(78)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 7")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(66)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 6")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(54)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 5")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(42)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 4")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(30)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 3")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(18)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 2")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(6)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("lte stress led show level 1")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetLteStressLED(-1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetLteStressLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set lte stress led off")



import logging
from ctypes import cdll, CDLL
from os import geteuid

logger = logging.getLogger(__name__)


class PSP:
    """
    PSP.

    sdk/include/lmbinc.h

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    """ Return Value """
    ERR_Success = 0
    ERR_Error = -1  # 0xFFFFFFFF
    ERR_NotExist = -2  # 0xFFFFFFFE
    ERR_NotOpened = -3  # 0xFFFFFFFD
    ERR_Invalid = -4  # 0xFFFFFFFC
    ERR_NotSupport = -5  # 0xFFFFFFFB
    ERR_BusyInUses = -6  # 0xFFFFFFFA
    ERR_BoardNotMatch = -7  # 0xFFFFFFF9
    ERR_DriverNotLoad = -8  # 0xFFFFFFF8
    """ IPMI access error """
    ERR_IPMI_IDLESTATE = -257  # 0xFFFFFEFF
    ERR_IPMI_WRITESTATE = -258  # 0xFFFFFEFE
    ERR_IPMI_READSTATE = -259  # 0xFFFFFEFD
    ERR_IPMI_IBF0 = -260  # 0xFFFFFEFC
    ERR_IPMI_OBF1 = -261  # 0xFFFFFEFB

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._liblmbio = None
        self._liblmbapi = None

    def __enter__(self) -> CDLL:
        if geteuid() != 0:
            raise PermissionError("Please uses root user !!!")
        self._liblmbio = cdll.LoadLibrary(self._lmb_io_path)
        self._liblmbapi = cdll.LoadLibrary(self._lmb_api_path)
        i_ret = self._liblmbapi.LMB_DLL_Init()
        if i_ret != self.ERR_Success:
            # This can happen when the BIOS message is different or the driver is not loaded.
            error_message = self.get_error_message("LMB_DLL_Init", i_ret)
            logger.error(error_message)
            raise self.PSPError(f"{error_message}, please confirm the API libraries is matched this platform")
        return self._liblmbapi

    def __exit__(self, exc_type, exc_val, exc_tb) -> bool:
        self._liblmbapi.LMB_DLL_DeInit()
        return False

    @classmethod
    def get_error_message(cls, function_name: str, error_code: int) -> str:
        message = f"{function_name}: 0x{error_code & 0xFFFFFFFF:08x}: "
        if error_code == cls.ERR_Error:
            message += "function failure"
        elif error_code == cls.ERR_NotExist:
            message += "library file not found or not exist"
        elif error_code == cls.ERR_NotOpened:
            message += "library not opened yet"
        elif error_code == cls.ERR_Invalid:
            message += "parameter invalid or out of range"
        elif error_code == cls.ERR_NotSupport:
            message += "this functions is not support of this platform"
        elif error_code == cls.ERR_BusyInUses:
            message += "busy"
        elif error_code == cls.ERR_BoardNotMatch:
            message += "the API library is not matched this platform"
        elif error_code == cls.ERR_DriverNotLoad:
            message += "the lmbiodrv driver or i2c-dev driver not loading"
        else:
            message += "unknown error"
        return message

    class PSPError(Exception):
        """Raised when any PSP error occurs."""

        def __init__(self, msg: str = "") -> None:
            self._message = msg

        def __repr__(self) -> str:
            return self._message

        __str__ = __repr__



import logging
import subprocess

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class ComPort:
    """
    COM Port.

    tool/gpio_config_tool.c

    :param config_tool_path: path of config_tool binary file
    """

    def __init__(self, config_tool_path: str = "/opt/lanner/psp/tool/config_tool"):
        self._config_tool_path = config_tool_path

    def set_com1_mode(self, mode: int) -> None:
        """Set COM1 mode.

        :param mode: 232/422/485
        """
        # Check type.
        if not isinstance(mode, int):
            raise TypeError("'mode' type must be int")
        # Check value.
        if mode not in (232, 422, 485):
            raise ValueError("'mode' value must be 232 or 422 or 485")
        # Set mode.
        result = subprocess.check_output(
            [self._config_tool_path, "-com1", f"-{mode}"]
        ).decode(encoding="utf-8").strip()
        # Check result.
        if result != "set muti function into gpio":
            error_message = "set com1 mode failure"
            logger.error(error_message)
            raise PSP.PSPError(error_message)
        logger.info(f"set com1 mode {mode}")

    def set_com1_termination(self, enable: bool) -> None:
        """Enable/Disable COM1 termination.

        :param enable: True = enable, False = disable
        """
        # Check type.
        if not isinstance(enable, bool):
            raise TypeError("'enable' type must be bool")
        if enable:
            # Enable termination.
            subprocess.check_output(
                [self._config_tool_path, "-com1", "-termon"]
            )
            logger.info("enable com1 termination")
        else:
            # Disable termination.
            subprocess.check_output(
                [self._config_tool_path, "-com1", "-termoff"]
            )
            logger.info("disable com1 termination")



import logging

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class WatchdogTimer:
    """
    Watchdog Timer.

    sdk/src_utils/sdk_sled/sdk_wdt.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path

    def enable(self, seconds: int) -> None:
        """Enable watchdog timer for specific seconds.

        :param seconds: seconds
        """
        # Check type.
        if not isinstance(seconds, int):
            raise TypeError("'seconds' type must be int")
        # Check value.
        if seconds <= 0:
            raise ValueError("'seconds' value must be > 0")
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_WDT_Config(seconds, 1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_WDT_Config", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_WDT_Start()
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_WDT_Start", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info(f"enable watchdog timer for {seconds:d} seconds")

    def disable(self) -> None:
        """Disable watchdog timer."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_WDT_Stop()
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_WDT_Stop", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("disable watchdog timer")

    def reset(self) -> None:
        """Reset watchdog timer."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_WDT_Tick()
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_WDT_Tick", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            logger.info("reset watchdog timer")



import logging
from ctypes import byref, c_uint8
from time import sleep

from .lmbinc import PSP

logger = logging.getLogger(__name__)


class SystemLED:
    """
    System/Status LED.

    sdk/src_utils/sdk_sled/sdk_sled.c

    :param lmb_io_path: path of liblmbio.so
    :param lmb_api_path: path of liblmbapi.so
    """

    def __init__(self,
                 lmb_io_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so",
                 lmb_api_path: str = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so") -> None:
        self._lmb_io_path = lmb_io_path
        self._lmb_api_path = lmb_api_path
        self._ub_read = c_uint8(0xFF)

    def off(self) -> None:
        """Set System/Status LED to off."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetSystemLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_GetSystemLED(byref(self._ub_read))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_GetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            if self._ub_read.value == 0:
                logger.info("set status led off")
            else:
                logger.warning("set status led failure")

    def green(self) -> None:
        """Set System/Status LED to green."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetSystemLED(1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_GetSystemLED(byref(self._ub_read))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_GetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            if self._ub_read.value == 1:
                logger.info("set status led green")
            else:
                logger.warning("set status led failure")

    def red(self) -> None:
        """Set System/Status LED to red/amber."""
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetSystemLED(2)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            i_ret = psp.LMB_SLED_GetSystemLED(byref(self._ub_read))
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_GetSystemLED", i_ret)
                logger.error(error_message)
                raise PSP.PSPError(error_message)
            if self._ub_read.value == 2:
                logger.info("set status led red/amber")
            else:
                logger.warning("set status led failure")

    def test(self, seconds: float = 2.0) -> None:
        """For testing (default 2 seconds delay).

        :param seconds: seconds
        """
        with PSP(self._lmb_io_path, self._lmb_api_path) as psp:
            i_ret = psp.LMB_SLED_SetSystemLED(1)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set status led green")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetSystemLED(2)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set status led red/amber")

            sleep(seconds)

            i_ret = psp.LMB_SLED_SetSystemLED(0)
            if i_ret != PSP.ERR_Success:
                error_message = PSP.get_error_message("LMB_SLED_SetSystemLED", i_ret)
                logger.error(error_message)
            else:
                logger.info("set status led off")
